/*
 * ultra_sonic.h
 *
 * Created: 13.3.2012 17:56:21
 *  Author: adam
 *
 * Driver pro ovldn ultrazvukovch idel SRF-05
 *
 */ 


#ifndef ULTRA_SONIC_H_
	#define ULTRA_SONIC_H_
	
	#define US_PIN PINE		// ultra sonic ports/pins aliases
	#define US_DDR DDRE
	#define US_PORT PORTE
	#define US_TRIG_C PE2	// 1-wire mode	// CENTER
	#define US_TRIG_R PE6	// 2-wire mode  // RIGHT
	#define US_ECHO_R PE5					
	#define US_TRIG_L PE4	// 2-wire mode  // LEFT
	#define US_ECHO_L PE3			

	#define SENSOR_LEFT 0
	#define SENSOR_CENTER 1
	#define SENSOR_RIGHT 2

	#define ERROR 0xFFFF	// error during waiting for some impulse 
	#define NO_OBJECT 0xFFFE

	WORD16 us_sensor(BYTE);
	WORD16 get_distance(BYTE);
	WORD16 distance_measure_left();
	WORD16 distance_measure_center();
	WORD16 distance_measure_right();
	void distance_print(WORD16, WORD16, WORD16);
	void delay();
	
	WORD16 us_sensor(BYTE site)
	{
		WORD16 d1;
		
		US_PORT = 0x00;		
		US_DDR = 0xFF;		// make PORTD output		
		delay();
		
		if(site == SENSOR_LEFT)
		{
			US_PORT = (1<<US_TRIG_L);		// make trigger impulse
			delay();
		}
		if(site == SENSOR_CENTER)
		{
			US_PORT = (1<<US_TRIG_C);		// make trigger impulse
			delay();
		}
		if(site == SENSOR_RIGHT)
		{
			US_PORT = (1<<US_TRIG_R);		// make trigger impulse
			delay();
		}
				
		US_PORT = 0x00;		// end of trigger impusle
		delay();
		US_DDR = 0x00;		
		delay();
				
		d1 = get_distance(site);	
		//_delay_ms(15);
		
		if(d1 == ERROR || d1 == NO_OBJECT)
			return d1;
		else
			return d1/58;
	}
	
	WORD16 get_distance(BYTE site)
	{
		WORD32 i;
		WORD16 result;
		//Wait for the rising edge
		if(site == SENSOR_LEFT)
		{
			for(i=0;i<200000;i++)
			{
				if(!(US_PIN & (1<<US_ECHO_L))) continue; else break;
			}
		}
		if(site == SENSOR_CENTER)
		{
			for(i=0;i<200000;i++)
			{
				if(!(US_PIN & (1<<US_TRIG_C))) continue; else break;
			}
		}
		if(site == SENSOR_RIGHT)
		{
			for(i=0;i<200000;i++)
			{
				if(!(US_PIN & (1<<US_ECHO_R))) continue; else break;
			}
		}		
		if(i>=200000)
			return ERROR;	//Indicates time out	
		//High Edge Found
		//Setup Timer1
		TCCR1A=0x00;
		TCCR1B=(1<<CS11);	//Prescaler = Fcpu/8
		TCNT1=0x00;			//Init counter
		//Now wait for the falling edge
		if(site == SENSOR_LEFT)
		{
			for(i=0;i<200000;i++)
			{
				if(US_PIN & (1<<US_ECHO_L))
				{
					if(TCNT1 > 60000) 
						break;	
					else 
						continue;
				}
				else
					break;
			}
		}
		if(site == SENSOR_CENTER)
		{
			for(i=0;i<200000;i++)
			{
				if(US_PIN & (1<<US_TRIG_C))
				{
					if(TCNT1 > 60000) 
						break; 
					else 
						continue;
				}
				else
					break;
			}
		}
		if(site == SENSOR_RIGHT)
		{
			for(i=0;i<200000;i++)
			{
				if(US_PIN & (1<<US_ECHO_R))
				{
					if(TCNT1 > 60000) 
						break; 
					else 
						continue;
				}
				else
					break;
			}
		}		
		if(i>=200000)
			return ERROR;	//Indicates time out
		//Falling edge found
		result=TCNT1;
		//Stop Timer
		TCCR1B=0x00;
		if(result > 60000)
			return NO_OBJECT;	//No obstacle
		else
			return (result>>1);		
	}			

	WORD16 distance_measure_left()
	{
		WORD16 distance_left[4];
		BYTE pocitadlo;
		BYTE i;
		distance_left[3] = 0;
		
		//return us_sensor(SENSOR_LEFT);  // prozatimn een bez potan prmru
	
		distance_left[0] = us_sensor(SENSOR_LEFT);
		_delay_ms(5);
		distance_left[1] = us_sensor(SENSOR_LEFT);
		_delay_ms(5);
		distance_left[2] = us_sensor(SENSOR_LEFT);
		_delay_ms(5);		
		//////////////////////////////////////////////////////////////////////////
		if(distance_left[0] == ERROR || distance_left[1] == ERROR || distance_left[2] == ERROR)
		{
			distance_left[3] = ERROR;
			return distance_left[3];
		}		
	
		if(distance_left[0] == NO_OBJECT && distance_left[1] == NO_OBJECT && distance_left[2] == NO_OBJECT)
		{
			distance_left[3] = NO_OBJECT;
			return distance_left[3];
		}				
		
		pocitadlo = 3;
		for(i = 0 ; i < 3 ; i++)
		{
			if(distance_left[i] == NO_OBJECT || distance_left[i] == ERROR ) 
				pocitadlo--;
			else
				distance_left[3] = distance_left[3] + distance_left[i];
		}	
		distance_left[3] = distance_left[3] / pocitadlo;				
		//////////////////////////////////////////////////////////////////////////
		return distance_left[3];
	}
	
	WORD16 distance_measure_center()
	{
		WORD16 distance_center[4];
		BYTE pocitadlo;
		BYTE i;
		distance_center[3] = 0;
		
		//return us_sensor(SENSOR_CENTER);  // prozatimn een bez potan prmru
		
		distance_center[0] = us_sensor(SENSOR_CENTER);
		_delay_ms(5);
		distance_center[1] = us_sensor(SENSOR_CENTER);
		_delay_ms(5);
		distance_center[2] = us_sensor(SENSOR_CENTER);
		_delay_ms(5);
		//////////////////////////////////////////////////////////////////////////
		if(distance_center[0] == ERROR || distance_center[1] == ERROR || distance_center[2] == ERROR)
		{
			distance_center[3] = ERROR;
			return distance_center[3];
		}		
	
		if(distance_center[0] == NO_OBJECT && distance_center[1] == NO_OBJECT && distance_center[2] == NO_OBJECT)
		{
			distance_center[3] = NO_OBJECT;
			return distance_center[3];
		}				
		
		pocitadlo = 3;
		for(i = 0 ; i < 3 ; i++)
		{
			if(distance_center[i] == NO_OBJECT || distance_center[i] == ERROR ) 
				pocitadlo--;
			else
				distance_center[3] = distance_center[3] + distance_center[i];
		}	
		distance_center[3] = distance_center[3] / pocitadlo;				
		//////////////////////////////////////////////////////////////////////////
		return distance_center[3];
	}
	
	WORD16 distance_measure_right()
	{
		WORD16 distance_right[4];
		BYTE pocitadlo;
		BYTE i;
		distance_right[3] = 0;
		
		//return us_sensor(SENSOR_RIGHT);  // prozatimn een bez potan prmru
		
		distance_right[0] = us_sensor(SENSOR_RIGHT);
		_delay_ms(5);
		distance_right[1] = us_sensor(SENSOR_RIGHT);
		_delay_ms(5);
		distance_right[2] = us_sensor(SENSOR_RIGHT);
		_delay_ms(5);	
		//////////////////////////////////////////////////////////////////////////
		if(distance_right[0] == ERROR || distance_right[1] == ERROR || distance_right[2] == ERROR)
		{
			distance_right[3] = ERROR;
			return distance_right[3];
		}		
	
		if(distance_right[0] == NO_OBJECT && distance_right[1] == NO_OBJECT && distance_right[2] == NO_OBJECT)
		{
			distance_right[3] = NO_OBJECT;
			return distance_right[3];
		}				
		
		pocitadlo = 3;
		for(i = 0 ; i < 3 ; i++)
		{
			if(distance_right[i] == NO_OBJECT || distance_right[i] == ERROR ) 
				pocitadlo--;
			else
				distance_right[3] = distance_right[3] + distance_right[i];
		}	
		distance_right[3] = distance_right[3] / pocitadlo;				
		//////////////////////////////////////////////////////////////////////////
		return distance_right[3];
	}
	
	void distance_print(WORD16 left, WORD16 center, WORD16 right)
	{
		lcd_gotoxy(0,3);
		lcd_puts("d1:    d2:    d3:   ");	
		//////////////////////////////////////////////////////////////////////////
		if(left == ERROR)
		{
			lcd_gotoxy(3,3);
			lcd_puts(" E ");			
		}
		if(left == NO_OBJECT)
		{
			lcd_gotoxy(3,3);
			lcd_puts("N/A");			
		}
		if(left != ERROR && left != NO_OBJECT)
		{
			lcd_gotoxy(3,3);
			lcd_write_int(left,3);
		}
		//////////////////////////////////////////////////////////////////////////
		if(right == ERROR)
		{
			lcd_gotoxy(17,3);
			lcd_puts(" E ");			
		}
		if(right == NO_OBJECT)
		{
			lcd_gotoxy(17,3);
			lcd_puts("N/A");			
		}
		if(right != ERROR && right != NO_OBJECT)
		{
			lcd_gotoxy(17,3);
			lcd_write_int(right,3);
		}
		//////////////////////////////////////////////////////////////////////////
		if(center == ERROR)
		{
			lcd_gotoxy(10,3);
			lcd_puts(" E ");			
		}
		if(center == NO_OBJECT)
		{
			lcd_gotoxy(10,3);
			lcd_puts("N/A");			
		}
		if(center != ERROR && center != NO_OBJECT)
		{
			lcd_gotoxy(10,3);
			lcd_write_int(center,3);
		}		
	}
	
	void delay()
	{
		for(WORD16 j = 0 ; j < 1 ; j++)
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");		
	}
	
#endif /* ULTRA_SONIC_H_ */